@@ -21,8 +21,11 @@ from .lib import Assets
 from .runners import (
     Krita,
     Blender,
+    AfterEffects,
+    AfterEffectsRender,
     VSCodium,
     NotepadPP,
+    MarkSequenceRunner,
     Firefox,
     SessionWorker,
     DefaultEditor,
@@ -33,7 +36,7 @@ from .site import (
     Synchronize, GotoCurrentSiteQueue,
     WorkingSites, ExchangeSites,
     MinioFileUploader, MinioFileDownloader,
-    RequestRevisionsAs,
+    RequestRevisions,
 )
 from .file import TrackedFile, Revision
 from .kitsu import KitsuAPIWrapper, KitsuBindings
@@ -193,7 +196,7 @@ class Synchronization(flow.Object):
     _project = flow.Parent()
     synchronize_files = flow.Child(SynchronizeFiles)
     show_jobs = flow.Child(GotoCurrentSiteQueue)
-    request_revisions_as = flow.Child(RequestRevisionsAs)
+    request_revisions = flow.Child(RequestRevisions)
 
     def summary(self):
         current_site = self._project.get_current_site()
@@ -599,8 +602,11 @@ class Project(flow.Object):
     def _register_runners(self):
         self._RUNNERS_FACTORY.ensure_runner_type(Blender)
         self._RUNNERS_FACTORY.ensure_runner_type(Krita)
+        self._RUNNERS_FACTORY.ensure_runner_type(AfterEffects)
+        self._RUNNERS_FACTORY.ensure_runner_type(AfterEffectsRender)
         self._RUNNERS_FACTORY.ensure_runner_type(VSCodium)
         self._RUNNERS_FACTORY.ensure_runner_type(NotepadPP)
+        self._RUNNERS_FACTORY.ensure_runner_type(MarkSequenceRunner)
         self._RUNNERS_FACTORY.ensure_runner_type(Firefox)
         self._RUNNERS_FACTORY.ensure_runner_type(Explorer)
         self._RUNNERS_FACTORY.ensure_runner_type(MinioFileUploader)
