@@ -13,10 +13,12 @@ import re
 from kabaret import flow
 from kabaret.flow_contextual_dict import ContextualView, get_contextual_dict
 from kabaret.subprocess_manager.flow import RunAction
-from .dependency import get_dependencies
 
+from ..utils.os import zip_folder, unzip_archive
+
+from .dependency import get_dependencies
 from .maputils import ItemMap, CreateGenericAction, ClearMapAction, RemoveGenericAction, SimpleCreateAction
-from libreflow.baseflow.runners import Runner, PythonRunner
+from .runners import Runner, PythonRunner
 
 
 class StaticSiteTypeChoices(flow.values.ChoiceValue):
@@ -247,11 +249,6 @@ class MinioDownloadFile(flow.Object):
             local_path
         )
 
-        if os.path.splitext(local_path)[1] == ".zip":
-            # Unzip
-            with zipfile.ZipFile(local_path, 'r') as zip_wc:
-                zip_wc.extractall(os.path.dirname(local_path))
-
 
 class Synchronize(ProcessJobs):
 
@@ -322,7 +319,23 @@ class Synchronize(ProcessJobs):
         server_path = server_path.replace("\\", "/")
 
         if job.type.get() == "Upload":
-            if not os.path.exists(local_path):
+            # Check if local_path refers to a folder (.zip format)
+            local_folder = os.path.dirname(local_path)
+            
+            if os.path.splitext(local_path)[1] == '.zip':
+                local_folder = os.path.dirname(local_path)
+                
+                if not os.path.exists(local_folder):
+                    # Return if folder does not exist
+                    self.root().session().log_error((
+                        "Following folder requested for upload but does not exist !"
+                        "\n>>>>> %s" % local_folder
+                    ))
+                    return
+                else:
+                    # Zip folder files
+                    zip_folder(local_folder, local_path)
+            elif not os.path.exists(local_path):
                 self.root().session().log_error((
                     "Following file requested for upload but does not exist !"
                     "\n>>>>> %s" % local_path
@@ -330,6 +343,10 @@ class Synchronize(ProcessJobs):
                 return
 
             self.uploader.upload(local_path, server_path)
+            
+            if os.path.splitext(local_path)[1] == '.zip':
+                os.remove(local_path)
+            
             self._set_sync_status(job.emitter_oid.get(), "Available", site_type="Exchange")
             self._touch(job.emitter_oid.get() + "/..")
         elif job.type.get() == "Download":
@@ -344,6 +361,12 @@ class Synchronize(ProcessJobs):
                 return
 
             self.downloader.download(server_path, local_path)
+            
+            # If it's a folder revision, extract its archive content
+            if os.path.splitext(local_path)[1] == '.zip':
+                unzip_archive(local_path, os.path.dirname(local_path))
+                os.remove(local_path)
+            
             self._set_sync_status(job.emitter_oid.get(), "Available")
             self._touch(job.emitter_oid.get() + "/..")
             self._touch(job.emitter_oid.get() + "/../../../..")
@@ -354,52 +377,108 @@ class Synchronize(ProcessJobs):
         self.root().project().synchronization.touch()
 
 
-class RequestedSiteChoiceValue(flow.values.ChoiceValue):
-    _revision = flow.Parent(2)
+class ActiveSiteChoiceValue(flow.values.SessionValue):
+    
+    DEFAULT_EDITOR = 'choice'
+    
+    _choices = flow.SessionParam(None).ui(editor='set')
 
     def choices(self):
-        choices = []
-        site_names = self.root().project().get_working_sites().mapped_names()
+        if self._choices.get() is None:
+            working_sites = self.root().project().get_working_sites()
+            names = working_sites.get_site_names(use_custom_order=True, active_only=True)
+            self._choices.set(names)
         
-        for site_name in site_names:
-            sync_status = self._revision.sync[site_name]
-            if sync_status.status.get() == 'Available':
-                choices.append(site_name)
-        
-        return choices
+        return self._choices.get()
 
 
-class RequestingSiteChoiceValue(flow.values.ChoiceValue):
+class ActiveSitesMultichoiceValue(ActiveSiteChoiceValue):
+    
+    DEFAULT_EDITOR = 'multichoice'
+
+
+class ActiveSiteRevisionAvailableChoiceValue(flow.values.ChoiceValue):
+    
+    DEFAULT_EDITOR = 'choice'
+    
+    _choices = flow.SessionParam(None).ui(editor='set')
     _revision = flow.Parent(2)
+    
+    exclude_choice = flow.SessionParam(None).ui(hidden=False)
 
     def choices(self):
-        choices = []
-        site_names = self.root().project().get_working_sites().mapped_names()
+        if self._choices.get() is None:
+            working_sites = self.root().project().get_working_sites()
+            choices = working_sites.get_site_names(use_custom_order=True, active_only=True)
+            exclude_choice = self.exclude_choice.get()
+            
+            if exclude_choice is not None:
+                try:
+                    choices.remove(exclude_choice)
+                except ValueError:
+                    pass
+            
+            choices_available = []
+            
+            for site_name in choices:
+                if self._revision.get_sync_status(site_name) == 'Available':
+                    choices_available.append(site_name)
+            
+            self._choices.set(choices_available)
         
-        for site_name in site_names:
-            sync_status = self._revision.sync[site_name]
-            if sync_status.status.get() == 'NotAvailable':
-                choices.append(site_name)
-        
-        return choices
+        return self._choices.get()
+
+
+class SiteSelection(flow.Object):
+    
+    source_site = flow.Param(None, ActiveSiteChoiceValue).watched()
+    target_site = flow.Param(None, ActiveSiteChoiceValue).watched()
+    
+    def child_value_changed(self, child_value):
+        if child_value in (self.source_site, self.target_site):
+            self.touch()
+    
+    def summary(self):
+        if self.source_site.get() == self.target_site.get():
+            return '⚠️ Source and target sites can\'t be the same !'
+        else:
+            return ''
 
 
 class RequestAs(flow.Action):
 
     _revision = flow.Parent()
+    sites = flow.Child(SiteSelection).ui(expanded=True)
     priority = flow.IntParam(50)
-    target_site = flow.Param(None, RequestingSiteChoiceValue).ui(label="Requesting site")
-    source_site = flow.Param(None, RequestedSiteChoiceValue).ui(label="Site to query")
     include_dependencies = flow.SessionParam(True).ui(editor='bool')
-
+    
     def get_buttons(self):
-        site_names = self.source_site.choices()
-        # Actually, an existing revision implies there is at least one
-        # site on which it is available, which is the source site
-        self.source_site.set(site_names[0])
+        self.sites.source_site.set(self.root().project().get_current_site().name())
+        target_site_choices = self.sites.target_site.choices()
+        
+        if not target_site_choices:
+            msg = self.message.get()
+            if msg is None:
+                msg = ''
+            
+            msg += (
+                '<h3><font color="#D5000D">Making requests is not possible since '
+                'there is no other site defined for this project.</font></h3>'
+            )
+            self.message.set(msg)
+            
+            return ['Cancel']
+        
+        self.sites.target_site.set(target_site_choices[0])
 
         return ["Request", "Cancel"]
     
+    def get_source_site_name(self):
+        return self.sites.source_site.get()
+    
+    def get_target_site_name(self):
+        return self.sites.target_site.get()
+    
     def allow_context(self, context):
         return (
             context
@@ -411,28 +490,29 @@ class RequestAs(flow.Action):
         if button == "Cancel":
             return
         
-        target_site = self.target_site.get()
+        source_site_name = self.get_source_site_name()
+        target_site_name = self.get_target_site_name()
 
         # Get requesting and requested sites
         sites = self.root().project().get_working_sites()
-        requesting_site = sites[target_site]
+        target_site = sites[target_site_name]
         
         # Add a download job for the requesting site
-        requesting_site.get_queue().submit_job(
+        target_site.get_queue().submit_job(
             job_type="Download",
             init_status="WAITING",
             emitter_oid=self._revision.oid(),
             user=self.root().project().get_user(),
-            studio=target_site,
+            studio=target_site_name,
             priority=self.priority.get(),
         )
-        self._revision.set_sync_status("Requested", site_name=target_site)
+        self._revision.set_sync_status("Requested", site_name=target_site_name)
         self._revision._revisions.touch()
         
         # Request dependencies if required
         if self.include_dependencies.get():
             request_deps = self._revision.request_dependencies
-            request_deps.target_site.set(self.target_site.get())
+            request_deps.target_site.set(target_site_name)
             request_deps.predictive_only.set(False)
             request_deps.run(None)
         
@@ -447,10 +527,10 @@ class RequestAs(flow.Action):
             self.root().project().synchronization.touch()
             return self.get_result()
 
-        requested_site = sites[self.source_site.get()]
+        source_site = sites[source_site_name]
 
         # Check if the source version upload is not already requested
-        for job in requested_site.get_jobs(type="Upload", status="WAITING"):
+        for job in source_site.get_jobs(type="Upload", status="WAITING"):
             if job.emitter_oid.get() == self._revision.oid():
                 self.root().session().log_warning(
                     "Revision already requested for upload in source site"
@@ -459,12 +539,12 @@ class RequestAs(flow.Action):
                 return self.get_result()
         
         # Add an upload job for the requested site
-        requested_site.get_queue().submit_job(
+        source_site.get_queue().submit_job(
             job_type="Upload",
             init_status="WAITING",
             emitter_oid=self._revision.oid(),
             user=self.root().project().get_user(),
-            studio=target_site,
+            studio=target_site_name,
             priority=self.priority.get(),
         )
 
@@ -476,14 +556,36 @@ class Request(RequestAs):
 
     _revision = flow.Parent()
     priority = flow.IntParam(50)
-    target_site = flow.Param("").ui(hidden=True)
+    source_site = flow.Param(None, ActiveSiteRevisionAvailableChoiceValue)
+    
+    sites = flow.Child(SiteSelection).ui(hidden=True)
 
     def get_buttons(self):
-        self.target_site.set(
-            self.root().project().get_current_site().name()
-        )
+        self.source_site.exclude_choice.set(self.root().project().get_current_site().name())
+        source_site_choices = self.source_site.choices()
+        
+        if not source_site_choices:
+            msg = self.message.get()
+            if msg is None:
+                msg = ''
+            
+            msg += (
+                '<h3><font color="#D5000D">Making requests is not possible since '
+                'the revision isn\'t available on any site.</font></h3>'
+            )
+            self.message.set(msg)
+            
+            return ['Cancel']
+        
+        self.source_site.set(source_site_choices[0])
 
-        return super(Request, self).get_buttons()
+        return ['Request', 'Cancel']
+    
+    def get_source_site_name(self):
+        return self.source_site.get()
+    
+    def get_target_site_name(self):
+        return self.root().project().get_current_site().name()
     
     def allow_context(self, context):
         return (
@@ -604,6 +706,8 @@ class Site(flow.Object):
     )
     configuration = flow.Child(ConfigureSite)
 
+    is_active = flow.BoolParam(True)
+
     def compute_child_value(self, child_value):
         if child_value is self.configured:
             self.configured.set(True)
@@ -626,6 +730,32 @@ class SiteMap(flow.Map):
     def short_names(self):
         return [s.short_name.get() for s in self.mapped_items()]
     
+    def get_site_names(self, use_custom_order=False, active_only=False, short_names=False):
+        if not use_custom_order:
+            sites = self.mapped_items()
+        else:
+            current_site = self.root().project().get_current_site()
+            unordered_names = self.mapped_names()
+            sites = []
+            
+            for name in current_site.get_ordered_site_names():
+                if self.has_mapped_name(name):
+                    sites.append(self.get_mapped(name))
+                    unordered_names.remove(name)
+            
+            for name in unordered_names:
+                sites.append(self.get_mapped(name))
+
+        if active_only:
+            sites = [s for s in sites if s.is_active.get()]
+        
+        if short_names:
+            names = [s.short_name.get() for s in sites]
+        else:
+            names = [s.name() for s in sites]
+        
+        return names
+    
     def _get_mapped_item_type(self, mapped_name):
         if mapped_name == "default":
             return self.mapped_type()
@@ -821,6 +951,8 @@ class SiteJobsPoolNames(flow.values.ChoiceValue):
 
 class WorkingSite(Site):
     
+    _site_map = flow.Parent()
+    
     site_type = flow.Param("Studio", StaticSiteTypeChoices)
     request_files_from_anywhere = flow.BoolParam(False).ui(
         tooltip=(
@@ -835,6 +967,8 @@ class WorkingSite(Site):
     root_linux_folder = flow.Param()
     root_darwin_folder = flow.Param()
     
+    ordered_site_names = flow.Param('')
+    
     sync_dl_max_connections = flow.IntParam(1)
     sync_up_max_connections = flow.IntParam(1)
     pool_names = flow.OrderedStringSetParam()
@@ -876,6 +1010,12 @@ class WorkingSite(Site):
                 )
             )
     
+    def get_ordered_site_names(self):
+        names_as_string = self.ordered_site_names.get()
+        names_as_string = names_as_string.replace(' ','')
+        
+        return names_as_string.split(',')
+    
     def get_queue(self):
         return self.queue.job_list
     
@@ -1012,49 +1152,63 @@ class SyncMap(flow.DynamicMap):
             row['Name'] += " (S)"
 
 
-class WorkingSiteChoiceValue(flow.values.ChoiceValue):
+class RequestedRevisions(flow.DynamicMap):
+    
+    STYLE_BY_STATUS = {
+        'Available': ('#45cc3d', ('icons.libreflow', 'blank')),
+        'Requested': ('#b9c2c8', ('icons.libreflow', 'exclamation-sign-colored')),
+        'NotAvailable': ('#cc3b3c', ('icons.libreflow', 'blank')),
+        'Undefined': ('#cc3b3c', ('icons.libreflow', 'unavailable')),
+    }
+    
+    request_action = flow.Parent()
+    oids = flow.SessionParam([]).ui(editor='set')
+    
+    def mapped_names(self, page_num=0, page_size=None):
+        return self.oids.get()
+    
+    def get_mapped(self, name):
+        if not self.has_mapped_name(name):
+            raise MappedNameError(self.oid(), name)
 
-    _parent = flow.Parent()
+        try:
+            obj = self._mng.get_object(name)
+        except ValueError:
+            raise
 
-    def choices(self):
-        return self.root().project().get_working_sites().mapped_names()
+        return obj
+    
+    def columns(self):
+        return ['Revision']
+    
+    def _fill_row_cells(self, row, item):
+        row['Revision'] = item.oid()
+    
+    def _fill_row_style(self, style, item, row):
+        status = 'Undefined'
+        
+        if hasattr(item, 'request_as'):
+            status = item.get_sync_status(site_name=self.request_action.sites.target_site.get())
+            
+        style['Revision_foreground-color'] = self.STYLE_BY_STATUS[status][0]
+        style['Revision_icon'] = self.STYLE_BY_STATUS[status][1]
 
 
-class RequestRevisionsAs(flow.Action):
+class RequestRevisions(flow.Action):
+    
+    ICON = ('icons.gui', 'share-option')
 
     pattern = flow.SessionParam("").watched().ui(
         placeholder="Revision oid pattern"
     )
-    source_site = flow.Param("default", WorkingSiteChoiceValue).watched()
-    target_site = flow.Param("default", WorkingSiteChoiceValue).watched()
-    revision_oids = flow.SessionParam("").ui(
-        editor="textarea",
-        html=True,
-        editable=False,
-    )
-
+    
+    sites = flow.Child(SiteSelection).ui(expanded=True)
+    revisions = flow.Child(RequestedRevisions).ui(expanded=True)
+    
     def allow_context(self, context):
         current_site = self.root().project().get_current_site()
         return context and current_site.request_files_from_anywhere.get()
     
-    def _title(self):
-        return "Request revisions"
-    
-    def child_value_changed(self, child_value):
-        if child_value in (self.target_site, self.source_site):
-            msg = "<h2>%s</h2>" % self._title()
-            if self.source_site.get() == self.target_site.get():
-                msg += (
-                    "<font color=#D5000D>"
-                    "Requested and requesting sites can't be the same."
-                    "</font>"
-                )
-
-            self.message.set(msg)
-        elif child_value is self.pattern:
-            self._oids = self._get_oid(self.pattern.get())
-            self.revision_oids.set("<br>".join(self._oids))
-    
     def _get_oid(self, pattern_str):
         oids = []
         patterns = []
@@ -1068,11 +1222,25 @@ class RequestRevisionsAs(flow.Action):
         return oids
 
     def get_buttons(self):
-        self.source_site.set(self.source_site.choices()[0])
-        self.revision_oids.set("")
-        self._oids = []
+        self.sites.source_site.set(self.root().project().get_current_site().name())
+        target_site_choices = self.sites.target_site.choices()
+        
+        if not target_site_choices:
+            msg = self.message.get()
+            if msg is None:
+                msg = ''
+            
+            msg += (
+                '<h3><font color="#D5000D">Making requests is not possible since '
+                'there is no other site defined for this project.</font></h3>'
+            )
+            self.message.set(msg)
+            
+            return ['Close']
+        
+        self.sites.target_site.set(target_site_choices[0])
 
-        return ["Request", "Close"]
+        return ["Request", "Refresh revision list", "Close"]
     
     def ls(self, root_oid):
         related_info, mapped_names = self.root().session().cmds.Flow.ls(root_oid)
@@ -1137,45 +1305,40 @@ class RequestRevisionsAs(flow.Action):
             oids += self.resolve_pattern(sub_oid)
         
         return oids
-
     
     def run(self, button):
         if button == "Close":
             return
-        
-        if not self._oids:
-            self._oids = self._get_oid(self.pattern.get())
-
-        objects = [self.root().get_object(oid) for oid in self._oids]
-        processed_oids = []
-        
-        for o in objects:
-            oid = o.oid()
-
-            if hasattr(o, 'request_as') and o.ready_for_sync.get():
-                source_site = self.source_site.get()
-                target_site = self.target_site.get()
-
-                if not target_site in o.request_as.target_site.choices():
-                    oid += f" <font color='red'>Available on {target_site}</font>"
-                elif not source_site in o.request_as.source_site.choices():
-                    oid += f" <font color='red'>Not available on {source_site}</font>"
-                else:
-                    o.request_as.source_site.set(source_site)
-                    o.request_as.target_site.set(target_site)
-                    o.request_as.run(None)
-                    oid += " <font color='green'>OK</font>"
-            else:
-                if not o.ready_for_sync.get():
-                    msg = "Revision not ready for sync"
-                else:
-                    msg = "Not a file revision"
-                
-                oid += " <font color='red'>%s</font>" % msg
+        elif button == "Refresh revision list":
+            self.revisions.oids.revert_to_default()
+            oids = self._get_oid(self.pattern.get())
+            self.revisions.oids.set(oids)
+            self.revisions.touch()
             
-            processed_oids.append(oid)
+            return self.get_result(close=False)
         
-        self.revision_oids.set("<br>".join(processed_oids))
+        source_site = self.sites.source_site.get()
+        target_site = self.sites.target_site.get()
+        
+        for obj in self.revisions.mapped_items():
+            # Skip objects which are not file revisions
+            if not hasattr(obj, 'request_as'):
+                continue
+            # Skip revisions which are not available on selected source site
+            if obj.get_sync_status(site_name=source_site) != 'Available':
+                continue
+
+            # Skip revisions which are already available on selected target site
+            status = obj.get_sync_status(site_name=target_site)
+            if status == 'Requested' or status == 'Available':
+                continue
+            
+            obj.request_as.sites.source_site.set(source_site)
+            obj.request_as.sites.target_site.set(target_site)
+            obj.request_as.run(None)
+            
+        
+        self.revisions.touch()
 
         return self.get_result(close=False)
 
